#
# Copyright (c) 2008 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
# 
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation. 
#
#
# Meta-package manager
#
# $Id$
#
# Author: Mihai Ibanescu <misa@redhat.com>

import os
import gzip
import cStringIO
import tempfile
from rhn.rpclib import xmlrpclib

import apt_inst
import apt_pkg
import apt.debfile

import struct

from types import ListType, TupleType, DictType

class InvalidPackageError(Exception):
    pass

def get_package_header(filename=None, file=None, fd=None):
    return load(filename=filename, file=file, fd=fd)[0]

# Loads an MPM and returns its header and its payload
def load(filename=None, file=None, fd=None):
    if (filename is None):
        raise ValueError, "filename has to be passed"
    if (filename is None and file is None and fd is None):
        raise ValueError, "No parameters passed"
    
    if filename is not None:
        f = open(filename)
    elif file is not None:
        f = file
    else: # fd is not None
        f = os.fdopen(os.dup(fd), "r")

    f.seek(0, 0)
    return load_deb(f, filename)

def load_deb(stream, filename):
    # Hmm, maybe deb
    
    try:
        import rhn_deb
    except ImportError:
        raise InvalidPackageError

    # Dup the file descriptor, we don't want it to get closed before we read
    # the payload
    newfd = os.dup(stream.fileno())
    stream = os.fdopen(newfd, "r")

    stream.flush()
    stream.seek(0, 0)
        
    try:
        #header = rhn_deb.get_package_header(file=stream)
        header = deb_Header(filename)
    except rhn_deb.InvalidPackageError, e:
        raise apply(InvalidPackageError, e.args)
    except rhn_deb.error, e:
        raise InvalidPackageError(e)
    stream.seek(0, 0)
    return header, stream
    
class deb_Header():
    "Wrapper class for an deb header - we need to store a flag is_source"
    def __init__(self, name):
        self.hdr = {}
        self.packaging = 'deb'
        self.signatures = []
        self.name = ''
        self.version = 0
        self.release = 0
        self.arch = 'all'
        self.is_source = 0
        
        try:
            pkg = apt_inst.DebFile(name)
            file = apt.debfile.DebPackage(filename=name)
        except Exception, e:
            raise InvalidPackageError(e)
       
        try:
            # Fill info about package
            control = pkg.control.extractdata("control")
            sections = apt_pkg.TagSection(control)
            self.name = sections["Package"]
            self.arch = sections["Architecture"] + '-deb'
            version = sections["Version"]
            version_tmpArr = version.split('-')
            if len(version_tmpArr) == 1:
                self.version = version
                self.release = "X"
            else:
                self.version = version_tmpArr[0]
                self.release = version_tmpArr[1]
        except Exception, e:
            raise InvalidPackageError(e)

    def getName(self):
        return self.name

    def getVersion(self):
        return self.version

    def getRelease(self):
        return self.release

    def getArch(self):
        return self.arch

    def is_signed(self):
        return 0
"""
    def __getitem__(self, name):
        return self.hdr.get(name)

    def __setitem__(self, name, item):
        self.hdr[name] = item

    def __delitem__(self, name):
        del self.hdr[name]

    def __getattr__(self, name):
        return getattr(self.hdr, name)
"""

class InvalidPackageError(Exception):
    pass
